import { middleLayerBase } from "./middleLayerBase";
import { buttonOnClick } from "./util";
import { initAir } from './air';

const { ccclass, property } = cc._decorator;

@ccclass
export default class NewClass extends middleLayerBase {


  routers = [];

  version = null;
  isDev = null;
  type = null;
  engine_uuid = null;
  async checkVersion() {
    const { versionName: onlineVersion, storeList } = await this.getOnlineVersion();
    const nativeVersion = await this.getNativeVersion();
    this.version = nativeVersion;
    if (onlineVersion != '' && onlineVersion != nativeVersion) {
      console.log('onlineVersion = ' + onlineVersion);
      console.log('nativeVersion = ' + nativeVersion);

      (<any>window).courseware.needUpdate('检测到更新', storeList[0]);
    }
  }
  async getNativeVersion() {
    const engineInfo = await this.getEngineInfo();
    const { version, isDev, type, uuid, mac } = JSON.parse(engineInfo);
    this.version = version;
    this.isDev = isDev;
    this.type = type;
    this.engine_uuid = mac || uuid;
    return this.version;
  }

  async getOnlineVersion() {
    let platform = '';
    if (cc.sys.isNative && cc.sys.os == cc.sys.OS_IOS) {
      platform = 'ios';
    } else if (cc.sys.isNative && cc.sys.os == cc.sys.OS_ANDROID) {
      platform = 'android';
    } else if (cc.sys.isNative && cc.sys.os == cc.sys.OS_WINDOWS) {
      platform = 'windows';
    } else {
      platform = 'web';
    }

    const res: any = await this.asyncCallNetworkApiGet('/api/application/v1/checkVersion', {
      pkg_name: 'com.iplayabc.ngt',
      platform: platform
    });
    return JSON.parse(res).data;
  }

  showLoading() {
    this.showWaitingLetters();
  }

  hideLoading() {
    this.hideWaitingLetters();
  }



  // 统一释放异步资源的方法
  onLoad() {
    initAir(this);
    this.reWriteAir();
  }

  async start() {
    this.node.zIndex = 9999;
    cc.game.addPersistRootNode(this.node);

    await this.checkVersion();

    if (this.isDev == 1) {
      cc.find(`middleLayer/debug`).active = true;
    }

    cc.find('middleLayer/BtnRight').active = false;
    cc.find('middleLayer/BtnLeft').active = false;
    cc.find('middleLayer/Page').active = false;
    cc.find('middleLayer/ExitBtn').active = false;

    
    // if (this.type == '') {
    //   await this.asyncCallNetworkApiGet(`/api/aixuebao/v1/engine/active`, {
    //     engineid: this.engine_uuid,
    //     product: 'ngt'
    //   });
    // }

    const str:any = await this.asyncCallNetworkApiGet(`/api/ngt/v1/engine/bindinfo`, {
      engine_id: this.engine_uuid
    });
    const resObj = JSON.parse(str);
    const sylles = resObj.data;
    if (sylles) {
      cc.sys.localStorage.setItem('activeSyllabuses', sylles);

      this.showWaitingLetters();
      this.loadOnlineBundle("cartoon_main");
    } else {
      cc.find(`middleLayer/activeLayer`).active = true;
    }

    this.initListener();
    this.onMiddleLayerLoadingFinish(); // 这个是用来隐藏原生端启动页的方法，酌情在适当的地方调用

  }

  async loadMainPage() {
    this.showLoading();
    const str:any = await this.asyncCallNetworkApiGet(`/api/ngt/v1/engine/bindinfo`, {
      engine_id: this.engine_uuid
    });
    this.hideLoading();
    const resObj = JSON.parse(str);
    const sylles = resObj.data;
    if (sylles) {
      cc.sys.localStorage.setItem('activeSyllabuses', sylles);
      this.showWaitingLetters();
      this.loadOnlineBundle("cartoon_main");
    } else {
      cc.find(`middleLayer/activeLayer`).active = true;
    }

  }

  initListener() {
    const exitBtn = cc.find('middleLayer/ExitBtn');
    buttonOnClick(exitBtn, ()=>{
      if (this.isShowWaiting) {
        return;
      }
      cc.find('middleLayer/BtnRight').active = false;
      cc.find('middleLayer/BtnLeft').active = false;
      cc.find('middleLayer/Page').active = false;
      cc.find('middleLayer/ExitBtn').active = false;

      this.showWaitingLetters();
      this.loadOnlineBundle("cartoon_main");
    }, 0.5)

    buttonOnClick(cc.find('middleLayer/BtnRight'), () => {
      if (this.isShowWaiting) {
        return;
      }
      this.goNextPage();
      if (this.courseIndex == 0) {
        cc.find('middleLayer/BtnLeft').active = false;
      } else {
        cc.find('middleLayer/BtnLeft').active = true;
      }
      if (this.courseIndex == this.courses.length - 1) {
        cc.find('middleLayer/BtnRight').active = false;
      } else {
        cc.find('middleLayer/BtnRight').active = true;
      }
    }, 0.5);

    buttonOnClick(cc.find('middleLayer/BtnLeft'), () => {
      if (this.isShowWaiting) {
        return;
      }
      this.goPrePage();
      if (this.courseIndex == 0) {
        cc.find('middleLayer/BtnLeft').active = false;
      } else {
        cc.find('middleLayer/BtnLeft').active = true;
      }
      if (this.courseIndex == this.courses.length - 1) {
        cc.find('middleLayer/BtnRight').active = false;
      } else {
        cc.find('middleLayer/BtnRight').active = true;
      }
    }, 0.5);

    buttonOnClick(cc.find('middleLayer/activeLayer/active'), async () => {
      if (this.isShowWaiting) {
        return;
      }
      this.showLoading();
      const input = cc.find('middleLayer/activeLayer/active_bg/active_input').getComponent(cc.EditBox);
      const activeCode = input.textLabel.string;
      const res:any = await this.asyncCallNetworkApiPost(`/api/ngt/v1/engine/bind`, {
        engine_id: this.engine_uuid, book_code: activeCode,
      });
      if (res && res.code == 200) {
        this.showTips("激活成功");
        cc.find('middleLayer/activeLayer').active = false;
        input.textLabel.string = "";
        this.hideLoading();
        await this.loadMainPage();

      } else {
        this.hideLoading();
        this.showTips(`激活失败：${res.msg}`);
      }
    });

    // 环境切换后门(两秒内连点十下以上)
    // middleLayer/backdoor
    const bk = cc.find(`middleLayer/backdoor`);
    bk.on('click', () => {
      const bk_count = bk["bk_count"];
      if (!bk_count) {
        bk.attr({bk_count: 1});
        setTimeout(() => {
          bk.attr({bk_count: null});
        }, 2000);
      } else if (bk_count>10){
        window["courseware"]?.changeHost();
      } else {
        bk.attr({bk_count: bk_count+1});
      }
    });

  }

  log(str) {
    const logStr = `${new Date().toLocaleString()}: ${str}`;
    console.log(logStr);
  }

  reWriteAir() {
    (<any>window).courseware.getData = (callback) => {
      let data = null;
      if (this.courseItem && this.courseItem.data) {
        data = JSON.parse(this.courseItem.data);
      }
      this.log("===成功调用getData===" + JSON.stringify(data));
      callback && callback(data);
    }
    (<any>window).courseware.nextPage = () => {
      this.goNextPage();
      if (this.courseIndex == 0) {
        cc.find('middleLayer/BtnLeft').active = false;
      } else {
        cc.find('middleLayer/BtnLeft').active = false;
      }
      if (this.courseIndex == this.courses.length - 1) {
        cc.find('middleLayer/BtnRight').active = false;
      } else {
        cc.find('middleLayer/BtnRight').active = true;
      }
    }
    (<any>window).courseware.beforePage = () => {
      this.goPrePage();
      if (this.courseIndex == 0) {
        cc.find('middleLayer/BtnLeft').active = false;
      } else {
        cc.find('middleLayer/BtnLeft').active = false;
      }
      if (this.courseIndex == this.courses.length - 1) {
        cc.find('middleLayer/BtnRight').active = false;
      } else {
        cc.find('middleLayer/BtnRight').active = true;
      }
    }
    if (!(<any>window).air) {
      (<any>window).air = {};
    }
    (<any>window).air.hideAirClassLoading = () => {
      console.log('汪汪汪');
      (<any>window).air.onCourseInScreen && (<any>window).air.onCourseInScreen(() => {
        console.log("***成功调用onCourseInScreen***");
        (<any>window).air.onCourseInScreen = null;
      });
      this.hideWaitingLetters();
    }
  }

  callNativeFunction(param) {
    const paramStr = JSON.stringify(param);
    if (cc.sys.isNative && cc.sys.os == cc.sys.OS_IOS) {
      return jsb.reflection.callStaticMethod('CocosMng', 'cocosWithNativeProtocol:', paramStr);
    } else if (cc.sys.isNative && cc.sys.os == cc.sys.OS_ANDROID) {
      return jsb.reflection.callStaticMethod('com/iplayabc/cocos/AppActivity', 'cocosWithNativeProtocol', '(Ljava/lang/String;)Ljava/lang/String;', paramStr);
    } else if (cc.sys.isNative && cc.sys.os == cc.sys.OS_WINDOWS) {
      console.log('汪汪汪')
    } else {
      console.error('非源生环境11');
    }
  }

  isShowWaiting = false;
  showWaitingLetters() {

    if (this.isShowWaiting) {
      return;
    }
    this.isShowWaiting = true;

    const layout = cc.find('middleLayer/layout');
    layout.removeAllChildren();

    const canvas = cc.find("Canvas");
    const bg = layout.addComponent(cc.Graphics);
    bg.fillRect(-canvas.width / 2, -canvas.height / 2, canvas.width, canvas.height);


    const laodingLayer = cc.find('middleLayer/loadingLayer');
    laodingLayer.active = true;

    const script = cc.find('middleLayer/loadingLayer/progress').getComponent('progressScript');
    script.setProgress(0);

    const id = setInterval(() => {
      script.setProgress(changeObj.value);
    }, 1);

    const changeObj = { value: 0 };
    cc.tween(changeObj)
      .to(5, { value: 0.9 })
      .call(() => {
        clearInterval(id);
      })
      .start();
  }

  hideWaitingLetters() {

    this.isShowWaiting = false;

    const layout = cc.find('middleLayer/layout');
    layout.active = false;

    const laodingLayer = cc.find('middleLayer/loadingLayer');
    laodingLayer.active = false;
  }
}
